// This is part of web maple 2.0beta

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;

public class WebMaple extends HttpServlet {
	private final String maple = "/opt/bin/maple";
	private final String webmaple = "/opt/bin/webmaple2pdf";

	//public void doGet(HttpServletRequest req, HttpServletResponse res)
	//	throws ServletException, IOException {
	//	doPost(req, res);
	//}

	public void doPost(HttpServletRequest req, HttpServletResponse res)
		throws ServletException, IOException
	{
		ServletOutputStream out = res.getOutputStream();
		Runtime r = Runtime.getRuntime();
		Process p;
		String plotIn;
		String gifOut;
		String pdfOut;
		String psOut;
		String textOut;
		String vrmlOut;
		String dxfOut;
		boolean inParse = false;

		// check host -- we need a different kind of authenstication
		//if ( !okClient(req, res) )
		//	return;

		// get parameters
		String mode = null;
		mode = req.getParameter("pdf");
		boolean doPdf = false;
		boolean doVrml = false;
		boolean doDxf = false;
		if ( (mode != null) && mode.equals("pdf") )
			doPdf = true;
		else if ( (mode != null) && mode.equals("vrml") )
			doVrml = true;
		else if ( (mode != null) && mode.equals("dxf") )
			doDxf = true;
		String mapleCommand = null;
		mapleCommand = req.getParameter("command");
		if ( mapleCommand == null ) {
			throwError(res, "No Maple Input");
			return;
		}

		//Need a new test for clean commands in the vrml mode.
		if ( doVrml && somethingUnsafeIn(mapleCommand) ) {
			throwError(res, "Can't run that command in vrml mode");
			return;
		}

		// Create command file for maple
		// This is dumb, but TempFile does not work
		// We need to create a TempDirectory
		String tmpDirectory = null;
		try {
			File tmpf = File.createTempFile("webmaple", ".d", new File("/tmp"));
			tmpDirectory = tmpf.getAbsolutePath();
			tmpf.delete();
			File tmpd = new File(tmpDirectory);
			boolean rv = tmpd.mkdir();
			if ( !rv ) {
				throwError(res, "File System Problem");
				if ( tmpDirectory != null )
					cleanUp(tmpDirectory);
				return;
			}
		} catch ( Exception e ) {
			throwError(res, "File System Problem");
			if ( tmpDirectory != null )
				cleanUp(tmpDirectory);
			return;
		}

		plotIn = tmpDirectory + "/webm";
		psOut = plotIn + ".ps";
		gifOut = plotIn + ".gif";
		pdfOut = plotIn + ".pdf";
		textOut = plotIn + ".txt";
		vrmlOut = plotIn + ".wrl";
		dxfOut = plotIn + ".dxf";

		// Builds the Maple input file
		try {
			PrintStream fileIn =
				new PrintStream(new FileOutputStream(new File(plotIn)));
			if ( doPdf ) {
				fileIn.println("plotsetup(cps,plotoutput=`" + psOut
					+ "`,plotoptions=`portrait,noborder`);");
			} else if ( doDxf ) {
				fileIn.println("plotsetup(dxf,plotoutput=`" + dxfOut
					+ "`);");
			} else if ( doVrml ) {
				fileIn.println("with(plottools):");
			} else
				fileIn.println("plotsetup(gif,plotoutput=`" + gifOut
					+ "`,plotoptions=`width=720,height=540`);");
			fileIn.println(mapleCommand);
			if ( doVrml )
				fileIn.println("vrml(fig3d,`" + vrmlOut + "`);");
			fileIn.close();
				
		} catch ( Exception e ) {
			throwError(res, "Plot Input file error in parse =" + inParse);
			cleanUp(tmpDirectory);
			return;
		}

		// Runs Maple on the input file
		try {
            String[] command = {"/bin/sh", "-c", maple + " -s -e0 -z < "
				+ plotIn + ">" + textOut + " 2>&1"};
			String[] command2 = {"/bin/sh", "-c", maple + " -s -e0 < "
					+ plotIn + ">" + textOut + " 2>&1"};
			if ( doVrml )
				command = command2;
            p = r.exec(command);
			p.waitFor();
			int status = p.waitFor();
			if ( status != 0 ) {
				throwError(res, "Maple error return: " + status);
				cleanUp(tmpDirectory);
				return;
			}
		} catch ( Exception e ) {
			throwError(res, "Maple output file error");
			cleanUp(tmpDirectory);
			return;
		}

		// Extra processing for Pdf files
		if ( doPdf )
			try {
				String[] command2 = {webmaple, plotIn};
				p = r.exec(command2);
				p.waitFor();
				int status = p.waitFor();
				if ( status != 0 ) {
					throwError(res, "WebMaple2pdf error return: " + status);
					cleanUp(tmpDirectory);
					return;
				}
			} catch ( Exception e ) {
				throwError(res, "WebMaple2PDF output file error");
				cleanUp(tmpDirectory);
				return;
			}

		// collect output
		File f = null;
		FileInputStream fileOut = null;
		boolean itsAGif = false;
		boolean itsAVrml = false;
		boolean itsADxf = false;
		if ( doPdf ) {
			f = new File(pdfOut);
			fileOut = null;
			try {
				fileOut = new FileInputStream(f);
			} catch ( Exception e ) {
				throwError(res, "No output");
				cleanUp(tmpDirectory);
				return;
			}
		} else if ( doDxf ) {
			f = new File(dxfOut);
			fileOut = null;
			try {
				fileOut = new FileInputStream(f);
				itsADxf = true;
			} catch ( Exception e ) {
				f = new File(textOut);
				try {
					fileOut = new FileInputStream(f);
				} catch ( Exception e2 ) {
					throwError(res, "Dxf: double not found error");
					cleanUp(tmpDirectory);
					return;
				}
			}
		} else if ( doVrml ) {
			f = new File(vrmlOut);
			fileOut = null;
			try {
				fileOut = new FileInputStream(f);
				itsAVrml = true;
			} catch ( Exception e ) {
				f = new File(textOut);
				try {
					fileOut = new FileInputStream(f);
				} catch ( Exception e2 ) {
					throwError(res, "Vrml: double not found error");
					cleanUp(tmpDirectory);
					return;
				}
			}
		} else {
			f = new File(gifOut);
			fileOut = null;
			try {
				fileOut = new FileInputStream(f);
				itsAGif = true;
			} catch ( Exception e ) {
				f = new File(textOut);
				try {
					fileOut = new FileInputStream(f);
				} catch ( Exception e2 ) {
					throwError(res, "Gif: double not found error");
					cleanUp(tmpDirectory);
					return;
				}
			}
		}

		long length = f.length();

		if ( length == 0 ) {
			throwError(res, "No output");
			cleanUp(tmpDirectory);
			return;
		}

		byte[] buffer = new byte[(int)length];
		try {
			fileOut.read(buffer);
		} catch ( Exception e ) {
			throwError(res, "pdf input file error");
			cleanUp(tmpDirectory);
			return;
		}

		// set output type
		if ( doPdf )
			res.setContentType("application/pdf");
		else if ( itsAVrml )
			res.setContentType("x-world/x-vrml");
		else if ( itsADxf )
			res.setContentType("vector/x-dxf");
		else if ( itsAGif )
			res.setContentType("image/gif");
		else
			res.setContentType("text/plain");
		setNoCache(req, res); 

		// send the output file
		try {
			out.write(buffer) ;
		} catch ( Exception e ) {
			cleanUp(tmpDirectory);
			return;
		}
		out.close();
		cleanUp(tmpDirectory);
	}

	// remove temporary files
	public void cleanUp(String tmpDirectory) {
		if ( tmpDirectory == null )
			return;

		String[] remove = { "/bin/rm", "-rf", tmpDirectory };
			//plotIn, psOut, pdfOut, gifOut, textOut };
		Process p;
		Runtime r;
		r = Runtime.getRuntime();

		try {
			p = r.exec(remove);
		} catch ( Exception e ) {
			// clean up failed.
		}
	}

	public String getServletInfo() {
		return "Web Maple 2.0 Servlet Front End";
	}

	private void throwError(HttpServletResponse res, String errorMsg)
			throws IOException {
		ServletOutputStream out = res.getOutputStream();
		res.setContentType("text/html");
		out.println("<HEAD><TITLE>Maple Error</TITLE></HEAD><BODY>");
		out.println("<h1>Maple Servlet Error:</h1>");
		out.println("<P>" + errorMsg + "<hr>");
		out.println("</BODY>");
		out.close();
	}

	// stolen from the web
	private void setNoCache(HttpServletRequest request, 
                                       HttpServletResponse response) {

       if(request.getProtocol().compareTo ("HTTP/1.0") == 0) {
           response.setHeader("Pragma", "no-cache");
       } else if (request.getProtocol().compareTo ("HTTP/1.1") == 0) {
           response.setHeader("Cache-Control", "no-cache");
       }
       response.setDateHeader("Expires", 0);
    }

	private String[] badWords = {
		"fopen", "!", "read", "save", "mkdir", "currentdir",
		"rmdir", "march", "system", "writeto", "appendto", "fprint"
	};

	// clean output from vrml commands
	private boolean somethingUnsafeIn(String in) {
		int k;
		for ( int i = 0; i < badWords.length; i++ ) {
			for ( int j = 0; j < in.length() - badWords[i].length() + 1; j++ ) {
				for ( k = 0; k < badWords[i].length(); k++ ) {
					if ( badWords[i].charAt(k) != in.charAt(j+k) )
						break;
				}
				if ( k == badWords[i].length() )
					return true;
			}
		}
		return false;
	}
}
